function sfun_pwmosciA(block)
% S-Funktion eines frequenz- und pulsweite-modulierbaren
% Rechteck-Oszillators
  setup(block);
end

%% -------------------------------------------------------------------
function setup(block)
  % Zahl der Anschlüsse
  block.NumInputPorts  = 2;
  block.NumOutputPorts = 0;
  
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  block.SampleTimes = [-1 0];

  % Callbacks
  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);
  block.RegBlockMethod('Start', @Start);
  block.RegBlockMethod('Update', @Update);
end

%% -------------------------------------------------------------------
function DoPostPropSetup(block)
  block.NumDworks = 1;    % Zeiger auf den Pulse Generator Block
  
  block.Dwork(1).Name            = 'BlockHandle';
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 0;      % double
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = false;
end

%% -------------------------------------------------------------------
function Start(block)
  % speichere einen Zeiger auf den Pulse Generator Block
  pulsBlock = find_system(gcs(), 'LookUnderMasks', 'all', ...
                         'BlockType','DiscretePulseGenerator');
  hPulse = get_param(pulsBlock{1}, 'Handle');
  block.Dwork(1).Data = hPulse;
end

%% -------------------------------------------------------------------
function Update(block)
  hPulse = block.Dwork(1).Data;
  f = block.InputPort(1).data;
  pw = block.InputPort(2).data;
  set_param(hPulse, 'Period', num2str(1/f));
  set_param(hPulse, 'PulseWidth', num2str(pw));
end
